import welink from "./platform/welink.js";

/**
 * 显示加载UI
 */
function showLoading(title = getApp().globalData.curPage.$t('common.loading'), mask = true) {
    uni.showLoading({
        title,
        mask
    })
}

/**
 * 显示消息提示框
 */
function showToast(title, duration = 2500, icon = "none") {
    uni.showToast({
        title,
        duration,
        icon
    })
}

/**
 * 简易模态框
 */
function showModal(content, confirm, showCancel = true, title = getApp().globalData.curPage.$t('common.warm-prompt')) {
    uni.showModal({
        title,
        content,
        showCancel,
        success(res) {
            if (res.confirm) {
                confirm()
            }
        }
    });
}

/**
 * 全参数模态框
 */
function showModal2(content,
    cancelText = Vue.prototype.$t('button.cancel'),
    confirmText = Vue.prototype.$t('button.confirm'),
    confirm,
    cancel) {
    if (confirm || cancel) {
        uni.showModal({
            title: getApp().globalData.curPage.$t('common.warm-prompt'),
            content,
            cancelText,
            confirmText
        }).then(res => {
            if (res[1].confirm) {
                confirm && confirm()
            } else if (res[1].cancel) {
                cancel && cancel()
            }
        }).catch(res => {
            uni.$kit.error(res)
        })
    } else {
        return uni.showModal({
            title: Vue.prototype.$t('common.warm-prompt'),
            content,
            cancelText,
            confirmText
        }).then(res => {
            return res[1]
        }).catch(res => {
            uni.$kit.error(res)
        })
    }
}

/**
 * 获取页面对象
 */
function getPage(upward = 1) { // 默认当前页面
    const pages = getCurrentPages();
    return pages[pages.length - upward];
}

/**
 * 获取当前页面对象
 */
function currentPage() {
    return getPage();
}

/**
 * 获取指定页面路由的webview对象，通过webview对象可控制页面的强大功能
 * @param {Object} routeName
 */
function getPageWebView(routeName) {
    const pages = getCurrentPages()
    for (let i = 0; i < pages.length; i++) {
        if (pages[i].route == routeName) {
            return pages[i].$getAppWebview()
        }
    }
    return null
}

/**
 * 获取当前页面的Webview对象
 */
function currentAppWebview() {
    return getApp().$getAppWebview();
}

/**
 * 回调页面的方法
 */
function callPage(upward = 1, method = 'updated', ...params) {
    const page = getPage(upward + 1)
    // #ifdef H5
    page && page[method] && page[method].apply(null, params)
    // #endif
    // #ifndef H5
    // #ifdef VUE2
    page && page.$vm[method] && page.$vm[method].apply(null, params)
    // #endif
    // #ifdef VUE3
    page && page.$scope[method] && page.$scope[method].apply(null, params)
    // #endif
    // #endif
}

/**
 * 回调上一个页面的方法
 */
function callParentPage(method = 'updated', ...params) {
    callPage(1, method, ...params)
}

/**
 * 跳转页面
 * @param {Object} jumpPath
 * @param {Object} options
 */
function navigateTo(jumpPath, options) {
    uni.navigateTo({
        url: (jumpPath.startsWith('http') || jumpPath.startsWith('/hybrid/html/pdf/web/viewer.html') || jumpPath.endsWith('.md')) ?
            (options && options.toUrlParams) ? `/pages/webview/webview${options.toUrlParams()}&url=${encodeURIComponent(jumpPath)}` :
            `/pages/webview/webview?url=${encodeURIComponent(jumpPath)}` : (options && options.toUrlParams) ? jumpPath + options
            .toUrlParams() : jumpPath
    })
}

/**
 * 跳转页面
 * @param {Object} jumpPath
 * @param {Object} options
 */
function switchTab(jumpPath, options) {
    // 切换
    uni.switchTab({
        url: jumpPath + ((options && options.toUrlParams) ? options.toUrlParams() : "")
    })
}

/**
 * 替换页面
 */
function redirectTo(jumpPath, options) {
    uni.redirectTo({
        url: jumpPath.startsWith('http') ? (options && options.toUrlParams) ?
            `/pages/webview/webview${options.toUrlParams()}&url=${encodeURIComponent(jumpPath)}` :
            `/pages/webview/webview?url=${encodeURIComponent(jumpPath)}` : (options && options.toUrlParams) ?
            jumpPath + options.toUrlParams() : jumpPath
    })
}

/**
 * 查询节点元素位置信息
 * @param {Object} selector
 */
async function getRect(selector) {
    return new Promise((resolve) => {
        uni.createSelectorQuery()
            .in(getApp().globalData.curPage)
            .select(selector)
            .fields({
                size: true, // 是否返回节点尺寸（width height）
                rect: true, // 是否返回节点布局位置（left right top bottom）
                scrollOffset: true // 是否返回节点的 scrollLeft scrollTop，节点必须是 scroll-view 或者 viewport
            }, res => resolve(res))
            .exec();
    })
}

/**
 * 查询节点的布局位置。相对于显示区域，以像素为单位。
 * @param {Object} selector
 */
async function getBoundingClientRect(selector, queryIn) {
    const selectors = selector.split(",");
    if (selectors.length == 1) {
        return new Promise((resolve) => {
            let query = uni.createSelectorQuery();
            if (queryIn) {
                query = query.in(queryIn);
            }
            query.select(selector)
                .boundingClientRect(res => resolve(res))
                .exec();
        })
    } else {
        return new Promise((resolve) => {
            let query = uni.createSelectorQuery();
            if (queryIn) {
                query = query.in(queryIn);
            }
            query.selectAll(selector)
                .boundingClientRect(res => resolve(res))
                .exec();
        })
    }
}

/*---------------------------H5----------------------------*/
// #ifdef H5
/**
 * 是否是微信浏览器
 */
function _isWeiXin() {
    var ua = window.navigator.userAgent.toLowerCase();
    return ua.match(/MicroMessenger/i) == 'micromessenger';
}

/**
 * 是否是PC浏览器
 */
function _isPc() {
    let userAgent = navigator.userAgent;
    let Agents = ["Android", "iPhone", "SymbianOS", "Windows Phone", "iPad", "iPod"];
    let flag = true;
    for (let v = 0; v < Agents.length; v++) {
        if (userAgent.indexOf(Agents[v]) > 0) {
            flag = false;
            break;
        }
    }
    return flag;
}

/**
 * 关闭H5页面
 */
function closePage() {
    var userAgent = navigator.userAgent;
    if (_isWeiXin()) {
        WeixinJSBridge.call('closeWindow');
    }
    if (!_isPc()) {
        window.opener = null;
        window.open('about:blank', '_self', '').close();
    }
    if (userAgent.indexOf("Firefox") != -1 || userAgent.indexOf("Chrome") != -1) {
        window.location.href = "about:blank";
    } else {
        window.opener = null;
        window.open("about:blank", "_self");
        window.close();
    }
}

/**
 * 导入远程JS文件
 * @param {Object} remotejs
 */
function importJS(remoteJS, position = 'body') {
    const script = document.createElement('script')
    script.type = 'text/javascript'
    script.src = remoteJS
    if (position == 'header') {
        document.header.appendChild(script);
    } else {
        document.body.appendChild(script);
    }
}
// #endif

/**
 * 上传文件
 * @param {Object} files
 * @param {Object} onProgress 上传进度回调函数
 */
function upload(files, onProgress) {
    return uni.$http.file.upload(
        files.map(item => { // formData传入文件名称和关联的业务对象
            return {
                fileName: item.name,
                bizId: item.bizId || "" // 如果上传文件时就要确定文件关联的业务对象，需要指定bizId
            }
        }),
        files,
        onProgress
    ).then(res => {
        if (uni.$kit.checkStatus(res, false)) {
            // 返回上传成功的文件ID列表
            return res.data.data.map(item => item.id);
        }
    }).catch(res => {
        uni.$kit.checkStatus(res, false);
        return res;
    })
}

/**
 * 扫描二维码
 * needResult： welink平台有效
 */
async function scanCode(needResult = 1) {
    // #ifndef H5
    return uni.scanCode({
        onlyFromCamera: false,
        scanType: ['qrCode', 'barCode']
    }).then(res => {
        uni.$kit.log('扫描结果: ', res[1]);
        return res[1];
    }).catch(res => {
        uni.$kit.error('扫描异常: ', res);
        return res;
    })
    // #endif
    // #ifdef H5
    switch (uni.$config.platform) {
        case "welink":
            return welink.scanCode(needResult);
        case "ewx":
        default:
            if (window.location.protocol.startsWith("https")) {
                return new Promise((resolve, reject) => {
                    uni.$on("scanResult", res => {
                        setTimeout(_ => {
                            resolve(res);
                        }, 100);
                    });
                    getApp().globalData.curPage.mixin_toPage("/platforms/h5/pages/common/scan-qrcode/scan-qrcode");
                });
            } else {
                showToast(getApp().globalData.curPage.$t('common.qrcode-scan-unsupported'));
            }
    }
    // #endif
}

/**
 * 开发环境
 */
function isDev() {
    return process.env.NODE_ENV == 'development'
}

/**
 * 生产环境
 */
function isProd() {
    return process.env.NODE_ENV == 'production'
}

/**
 * 平台类型
 */
function platform() {
    return uni.getSystemInfoSync().platform;
}

/**
 * Android平台
 */
function isAndroid() {
    return uni.getSystemInfoSync().platform === "android";
}

/**
 * IOS平台
 */
function isIos() {
    return uni.getSystemInfoSync().platform === "ios";
}

/**
 * 是否是移动端
 */
function isMobile() {
    let sUserAgent = navigator.userAgent.toLowerCase()
    let bIsMobile = /AppleWebKit.*Mobile/i.test(navigator.userAgent)
    let bIsIpad = sUserAgent.match(/ipad/i) !== null
    let bIsIphoneOs = sUserAgent.match(/iphone os/i) !== null
    let bIsMidp = sUserAgent.match(/midp/i) !== null
    let bIsUc7 = sUserAgent.match(/rv:1.2.3.4/i) !== null
    let bIsUc = sUserAgent.match(/ucbrowser/i) !== null
    let bIsAndroid = sUserAgent.match(/android/i) !== null
    let bIsCE = sUserAgent.match(/windows ce/i) !== null
    let bIsWM = sUserAgent.match(/windows mobile/i) !== null

    if (bIsMobile ||
        bIsIpad ||
        bIsIphoneOs ||
        bIsMidp ||
        bIsUc7 ||
        bIsUc ||
        bIsAndroid ||
        bIsCE ||
        bIsWM) {
        return true
    } else {
        return false
    }
}

export default {
    // UI相关
    showLoading,
    showToast,
    showModal,
    showModal2,

    // 页面相关
    getPage,
    currentPage,
    getPageWebView,
    currentAppWebview,
    callPage,
    callParentPage,

    // 页面跳转
    navigateTo,
    switchTab,
    redirectTo,

    // 节点信息查询
    getRect,
    getBoundingClientRect,

    // #ifdef H5
    closePage,
    importJS,
    // #endif

    // 网络请求
    upload,
    
    // 设备能力接口
    scanCode,
    
    // env
    isDev,
    isProd,
    platform,
    isAndroid,
    isIos,
    isMobile
}
