/**
 * 鉴权相关，登录和登出处理
 */
export default {
    methods: {
        /**
         * 用户登陆，兼容第三方集成平台免登陆
         */
        mixin_login(showLoading = true) {
            // #ifndef H5
            this.mixin_tokenLogin(showLoading);
            // #endif

            // #ifdef H5
            switch (this.$config.platform) {
                case "welink":
                    this.mixin_welinLogin();
                    break;
                default:
                    this.mixin_tokenLogin(showLoading);
                    break;
            }
            // #endif
        },

        /**
         * 用户名密码登录
         * @param {String} userName
         * @param {String} password
         * @param {Boolean} showLoading
         */
        mixin_pwdLogin(userName, password, showLoading = true) {
            if (this.$config.uniCloud) {
                const uniIdCo = uniCloud.importObject("uni-id-co", {
                    errorOptions: {
                        type: 'toast'
                    }
                })
                let data = {
                    "password": password,
                    // "captcha": this.captcha
                }

                if (userName.isMobile()) {
                    data.mobile = userName
                } else if (userName.isEmail()) {
                    data.email = userName
                } else {
                    data.username = userName
                }

                uniIdCo.login(data).then(res => {
                    const {
                        token,
                        tokenExpired
                    } = res.newToken;
                    this.mixin_loginSuccess({}, token, tokenExpired);
                }).catch(err => {
                    if (err.errCode == 'uni-id-captcha-required') {
                        this.mixin_showModal("没有验证码");
                    }
                })
            } else {
                this.mixin_http({
                    method: this.$http.user.loginByPassword,
                    param: [userName, password]
                }, null, showLoading).then(res => {
                    this.__loginThen(res, userName);
                }).catch(this.mixin_loginCatch);
            }
        },

        /**
         * 短信验证码登录
         * @param {String} mobile
         * @param {String} vcode
         * @param {Boolean} showLoading
         */
        mixin_vcodeLogin(mobile, vcode, showLoading = true) {
            this.mixin_http({
                method: this.$http.user.loginByVcode,
                param: [mobile, vcode]
            }, null, showLoading).then(res => {
                this.__loginThen(res, this.vuex_user.userInfo.loginName);
            }).catch(this.mixin_loginCatch);
        },

        /**
         * Token登录
         * @param {Boolean} showLoading
         */
        mixin_tokenLogin(showLoading = true) {
            this.mixin_http({
                method: this.$http.user.loginByToken
            }, null, showLoading).then(res => {
                this.__loginThen(res, this.vuex_user.userInfo.loginName, true);
            }).catch(res => {
                this.mixin_loginCatch(res);
                // #ifndef MP
                // 如果没有登录过则跳转到登录页面
                this.mixin_redirectTo(this.mixin_getLoginUrl());
                // #endif
                // #ifdef MP
                this.mixin_redirectTo(this.mixin_homePage);
                // #endif
            })
        },

        /**
         * 登录then处理
         * @param {Object} res
         * @param {Object} userName
         */
        __loginThen(res, userName, toLoginPage = false) {
            if (this.$kit.checkStatus(res, false)) {
                const userInfo = res.data.data;
                userInfo.loginName = userName;
                // 登录成功
                this.mixin_loginSuccess(userInfo, userInfo.token, userInfo.tokenExpired);
            } else {
                this.mixin_loginFailed(res);
                if (toLoginPage) {
                    // 如果没有登录过则跳转到登录页面
                    this.mixin_redirectTo(this.mixin_getLoginUrl());
                }
            }
        },

        /**
         * 登录成功，保存用户信息和token
         * @param {Object} userInfo
         * @param {String} token
         * @param {Number} tokenExpired
         */
        mixin_loginSuccess(userInfo, token, tokenExpired) {
            // 保存用户数据
            this.$u.vuex('vuex_user', {
                userInfo,
                token,
                tokenExpired
            })

            // 支持uni-id体系的token管理，保存token标志
            uni.setStorageSync('userInfo', userInfo); // vuex state 管理中，user空间使用userInfo获取用户信息
            uni.setStorageSync('uni_id_token', token);
            uni.setStorageSync('uni_id_token_expired', tokenExpired);

            // 跳转首页面
            this.mixin_reLaunchHomePage();
        },

        /**
         * 登录成功，跳转首页面
         */
        mixin_reLaunchHomePage() {
            let redirectUrl = this.mc_options.uniIdRedirectUrl || this.mixin_homePage;
            if (redirectUrl) {
                redirectUrl = redirectUrl.startsWith("/") ? redirectUrl : `/${redirectUrl}`;
            }
            this.mixin_reLaunch(redirectUrl);
        },

        /**
         * 登录失败处理，可重写
         * @param {Object} res
         */
        mixin_loginFailed(res) {
            this.mixin_showToast(res.data.message);
        },

        /**
         * 登录异常处理，可重写
         * @param {Object} res
         */
        mixin_loginCatch(res) {
            this.mixin_showToast(this.$t('login.exception'));
        },

        // #ifdef H5
        /**
         * welink免登
         */
        mixin_welinLogin() {
            if (document.readyState == 'complete') {
                this.mixin_welinkDoLogin()
            } else {
                document.onreadystatechange = _ => {
                    if (document.readyState == 'complete') {
                        this.mixin_welinkDoLogin()
                    }
                }
            }
        },

        /**
         * welink登录
         */
        mixin_welinkDoLogin() {
            HWH5.getAuthCode().then(data => {
                this.mixin_httpCall({
                    method: this.$http.user.loginByPlatform,
                    param: ['welink', data.code, this.vuex_tenantId]
                }).then(res => {
                    if (this.$kit.checkStatus(res)) {
                        // 登录成功
                        this.mixin_loginSuccess(res.data.data);
                    } else {
                        this.mixin_loginFailed(res);
                    }
                }).catch(res => {
                    this.mixin_loginCatch(res);
                })
            }).catch(error => {
                this.mixin_loginCatch(res);
            });
        },
        // #endif

        /**
         * 清除缓存
         */
        mixin_clearCache() {
            // 清除用户数据
            this.$u.vuex('vuex_user', {
                userInfo: null,
                tokenExpired: 0
            })

            // 支持uni-id体系的token管理，清除token标志
            uni.setStorageSync('userInfo', {});
            uni.removeStorageSync('uni_id_token');
            uni.setStorageSync('uni_id_token_expired', 0);

            // 跳转到登录页面
            setTimeout(_ => this.mixin_redirectTo(this.mixin_getLoginUrl()), 300);
        },

        /**
         * 退出登录
         */
        async mixin_logout() {
            this.mixin_showLoading();
            if (this.$config.uniCloud) {
                if (uniCloud.getCurrentUserInfo().tokenExpired > Date.now()) {
                    try {
                        await uniCloud.importObject("uni-id-co").logout();
                    } catch (e) {
                        console.error(e);
                    }
                }
            }
            this.mixin_clearCache();
            uni.hideLoading();
        }
    }
}
