/**
 * 实现简单版本的列表页面的基本组件
 */
import mixinBase from '../request/base.js'

export default {
    /**
     * 混入列表更新、下拉刷新上拉加载功能
     */
    mixins: [mixinBase],

    /**
     * 数据
     */
    data() {
        return {
            // 分页请求的当前页数
            mv_current: 0,
            // loadmore控件的状态，loadmore/loading/nomore
            mv_status: 'loading',
            // 已加载数据
            mv_loaded: false,
            // 列表数据
            mv_data: [],
            // 检索变量，可与cu-search-bar使用，v-model="mv_key"
            mv_key: "",
            // 自动检索
            mv_autoSearch: true
        }
    },
    
    watch: {
        /**
         * 监控mv_key属性的变化，获取检索后的业务数据
         * @param {Object} newValue
         * @param {Object} oldValue
         */
        mv_key(newValue, oldValue) {
            // 手动检索
            if (!this.mv_autoSearch) {
                return;
            }
            // 检索之前做处理（如：本地检索），返回true则不再请求网络数据
            if (this.mixin_searchBefore && this.mixin_searchBefore()) {
                return;
            }
            // 重新请求数据
            this.mixin_reloadData();
        }
    },

    methods: {
        /**
         * 添加页面的添加动作完成后回调
         */
        mixin_added() {
            // 强制更新当前页面
            this.mv_updatePage = true
            
            // 更多操作通过mixin_doAdded实现
            if (this.mixin_doAdded) {
                this.mixin_doAdded();
            }
        },
        
        /**
         * 添加页面的更新动作完成后回调
         */
        mixin_updated(updateItem) {
            if (updateItem) { // 直接更新data中的数据
                for (let i = 0; i < this.mv_data.length; i++) {
                    const item = this.mv_data[i];
                    if (item.id == updateItem.id) {
                        const newItem = { ...item, ...updateItem }
                        this.mv_data.splice(i, 1, this.mixin_handleItem ? this.mixin_handleItem(newItem): newItem);
                        break;
                    }
                }
            } else {
                // 强制更新当前页面
                this.mv_updatePage = true
            }
        },
        
        /**
         * 详情页面的删除动作完成后回调
         */
        mixin_deleted(delId) {
            if (delId) {
                for (let i = 0; i < this.mv_data.length; i++) {
                    if (this.mv_data[i].id == delId) {
                        this.mv_data.splice(i, 1);
                        break;
                    }
                }
            } else {
                this.mv_updatePage = true;
            }
        },
        
        /**
         * 判断是否还有更多业务数据
         */
        mixin_hasMoreData() {
            return this.mv_status == 'loading'
        },
        
        /**
         * 下拉刷新时重置页面数据
         */
        mixin_resetForRefresh() {
            this.mixin_resetPage();
        },
        
        /**
         * 更新数据时重置数据
         */
        mixin_resetForUpdate() {
            this.mixin_resetPage();
        },
        
        /**
         * 重置列表参数
         */
        mixin_resetPage() {
            this.mv_current = 0
            this.mv_status = 'loading'
            this.mv_loaded = false
        },
        
        /**
         * scrollview滑动到底部加载更多数据
         */
        mixin_onScrollToLower() {
            if (!this.mv_loading && this.mixin_hasMoreData()) {
                this.mixin_loadData(false)
            }
        },
        
        /**
         * 处理列表数据项
         * @param {Object} item
         */
        mixin_handleItem(item) {
            return item;
        },
        
        /**
         * 格式化数据
         * @param {Object} res
         */
        mixin_handleData(res) {
            return res.data.data.map(item => this.mixin_handleItem(item))
        },
        
        /**
         * 请求结束，可能是异常，该方法主要用于设置相关标志和隐藏加载视图
         */
        mixin_reqEnd(showLoading = true, exception = false) {
            this.mv_loaded = true
            
            // 如果是异常
            if (exception) {
                // #ifdef APP-NVUE
                this.mixin_list_reqEnd(false)
                // #endif
                // #ifndef APP-NVUE
                this.mv_status = 'erro'
                // #endif
            }
            
            // 下拉加载不会显示modal加载框，但上拉加载更多也不会显示，此时不需要uni.stopPullDownRefresh，不多调用也不影响页面逻辑
            if (showLoading) {
                this.mv_loading = false
                uni.hideLoading()
                // #ifdef APP-NVUE
                this.mixin_list_reqEnd(true)
                // #endif
            } else {
                // #ifdef APP-NVUE
                this.mixin_list_reqEnd(true)
                // #endif
                // #ifndef APP-NVUE
                uni.stopPullDownRefresh()
                // #endif
            }
        },

        /**
         * 业务数据请求返回正常处理
         */
        mixin_reqThen(res, showLoading = false) {
            if (this.$kit.checkStatus(res, false)) {
                if (this.mv_current == 0) {
                    this.mv_data = this.mixin_handleData(res);
                } else {
                    this.mv_data = [ ...this.mv_data, ...this.mixin_handleData(res) ];
                }
                
                if (this.$kit.isArray(res)) {
                    if (!res.map(item => this.mixin_noMoreData(item.data.data)).includes(false)) {
                        this.mv_status = "nomore";
                    }
                } else if (this.mixin_noMoreData(res.data.data)) {
                    this.mv_status = "nomore";
                }
                this.mv_current += 1
            } else {
                this.mv_status = "erro";
            }
            
            this.mixin_reqEnd(showLoading)
        }
    }
}
