/**
 * 带有底部tabbar和tab的列表页面的基本组件
 */
import mixinBase from '../request/base.js'

export default {
    /**
     * 混入列表更新、下拉刷新上拉加载功能
     */
    mixins: [mixinBase],

    /**
     * 数据
     */
    data() {
        return {
            // 当前tabbar的索引
            mv_barIndex: 0,
            // tabbar数据
            mv_tabbars: [],
            // 是否根据tabbar索引变化自动更新页面标题
            mv_autoTitle: false
        }
    },

    computed: {
        /**
         * 当前标签
         */
        curTabbar() {
            return this.mv_tabbars[this.mv_barIndex]
        }
    },

    /**
     * 注意：watch方法类似周期钩子方法，混入规则：组合为数组，并先于组件方法运行
     */
    watch: {
        /**
         * 监控tabIndex属性的变化，获取检索后的业务数据
         * @param {Object} newValue
         * @param {Object} oldValue
         */
        mv_barIndex(newValue, oldValue) {
            // 自动更新页面标题为tabbar项的text
            if (this.mv_autoTitle) {
                this.navTitle = this.curTabbar.text;
            }
            // 切换tabbar，返回false表示继续执行，返回true表示处理完成不在继续执行
            if (this.mixin_onSwitch()) {
                return;
            } 
            // 加载数据
            if (!this.curTabbar.loaded) {
                this.mixin_reloadData(true)
            }
        }
    },

    methods: {
        /**
         * 切换tab，返回false表示继续执行，返回true表示处理完成不在继续执行
         */
        mixin_onSwitch() {
            return false;
        },
        
        /**
         * 添加页面的添加动作完成后回调
         */
        mixin_doAdded(barIndex) {
            if (!this.$kit.isStrictEmpty(barIndex)) {
                this.mv_barIndex = barIndex
            }
        },
        
        /**
         * 添加页面的更新动作完成后回调
         */
        mixin_doUpdated(target) {
            for (let i = 0; i < this.curTabbar.data.length; i++) {
                const item = this.curTabbar.data[i];
                if (item.id == target.id) {
                    const newItem = { ...item, ...target }
                    this.curTabbar.data.splice(i, 1, this.mixin_handleItem ? this.mixin_handleItem(newItem): newItem);
                    break;
                }
            }
        },
        
        /**
         * 详情页面的删除动作完成后回调
         */
        mixin_doDeleted(itemId) {
            for (let i = 0; i < this.curTabbar.data.length; i++) {
                if (this.curTabbar.data[i].id == itemId) {
                    this.curTabbar.data.splice(i, 1);
                    break;
                }
            }
        },
        
        /**
         * 判断是否还有更多业务数据
         */
        mixin_hasMoreData() {
            return this.curTabbar.status == 'loading'
        },
        
        /**
         * 下拉刷新时重置页面数据
         */
        mixin_resetForRefresh() {
            this.mixin_resetCurTabbar();
        },
        
        /**
         * 更新数据时重置数据
         */
        mixin_resetForUpdate() {
            this.mixin_resetCurTabbar();
        },

        /**
         * 切换tabbar
         */
        mixin_onTabbarChanged(index) {
            this.mv_barIndex = index
        },

        /**
         * tabbar可能会和swiper一起使用
         * @param {Object} e
         */
        mixin_onSwiperChanged(e) {
            this.mixin_onTabbarChanged(e.detail.current)
        },

        /**
         * scrollview滑动到底部加载更多数据
         */
        mixin_onScrollToLower() {
            if (!this.mv_loading && this.mixin_hasMoreData()) {
                this.mixin_loadData(false)
            }
        },

        /**
         * 重置列表参数
         */
        mixin_resetPage() {
            this.mv_tabbars.forEach(tabbar => {
                tabbar.current = 0
                tabbar.status = 'loading'
                tabbar.loaded = false
            })
        },

        /**
         * 重置当前页签列表参数
         */
        mixin_resetCurTabbar() {
            this.curTabbar.current = 0
            this.curTabbar.status = 'loading'
            this.curTabbar.loaded = false
        },

        /**
         * 重新加载业务数据
         */
        mixin_reloadData(showLoading = true) {
            this.mixin_resetCurTabbar()
            this.mixin_loadData(showLoading)
        },

        /**
         * 请求结束，可能是异常，该方法主要用于设置相关标志和隐藏加载视图
         */
        mixin_reqEnd(showLoading = true, exception = false) {
            this.curTabbar.loaded = true

            // 如果是异常
            if (exception) {
                // #ifdef APP-NVUE
                this.mixin_tabbarlist_reqEnd(false)
                // #endif
                // #ifndef APP-NVUE
                this.curTabbar.status = 'erro'
                // #endif
            }

            // 下拉加载不会显示modal加载框，但上拉加载更多也不会显示，此时不需要uni.stopPullDownRefresh，不多调用也不影响页面逻辑
            if (showLoading) {
                this.mv_loading = false
                uni.hideLoading()
                // #ifdef APP-NVUE
                this.mixin_tabbarlist_reqEnd(true)
                // #endif
            } else {
                // #ifdef APP-NVUE
                this.mixin_tabbarlist_reqEnd(true)
                // #endif
                // #ifndef APP-NVUE
                uni.stopPullDownRefresh()
                // #endif
            }
        },

        /**
         * 格式化数据项
         * @param {Object} item
         */
        mixin_handleItem(item) {
            return item;
        },

        /**
         * 格式化数据
         * @param {Object} res
         */
        mixin_handleData(res) {
            return res.data.data.map(item => this.mixin_handleItem(item))
        },

        /**
         * 业务数据请求返回正常处理，默认使用net处理返回的数据，可在页面重写该方法
         */
        mixin_reqThen(res, showLoading = true) {
            if (this.$kit.checkStatus(res, false)) {
                if (this.curTabbar.mv_current == 0) {
                    this.curTabbar.mv_data = this.mixin_handleData(res);
                } else {
                    this.curTabbar.mv_data = [ ...this.curTabbar.mv_data, ...this.mixin_handleData(res) ];
                }
                
                if (this.$kit.isArray(res)) {
                    if (!res.map(item => this.mixin_noMoreData(item.data.data)).includes(false)) {
                        this.curTabbar.mv_status = "nomore";
                    }
                } else if (this.mixin_noMoreData(res.data.data)) {
                    this.curTabbar.mv_status = "nomore";
                }
                this.curTabbar.mv_current += 1
            } else {
                this.curTabbar.mv_status = "erro";
            }
            
            this.mixin_reqEnd(showLoading)
        },

        /**
         * 加载业务数据
         */
        mixin_loadData(showLoading = true) {
            /**
             * 其他准备工作，如登录检查、权限控制等
             */
            if (!this.mixin_loadBefore()) {
                return
            }

            // 如果有正在执行的任务，不会重复执行
            if (this.mv_loading) {
                return
            }

            if (!this.mixin_request) {
                return
            }

            if (showLoading) {
                this.mv_loading = true
                this.$kit.showLoading()
            }

            this.mixin_httpCall(this.mixin_request).then(res => {
                this.mixin_reqThen(res, showLoading)
            }).catch(res => {
                this.mixin_reqCatch(res, showLoading)
            })
        }
    }
}
