/**
 * 简化页面常用操作，全局混入所有页面
 */
import mixinCommon from "./common/common.js";
import mixinJump from "./common/jump.js";
// #ifdef APP-NVUE
import mixinNvue from './nvue.js';
// #endif
// #ifndef MP-WEIXIN
import mixinVConsole from './vconsole.js'
// #endif
import mixinRotateScreen from './rotate-screen.js';
import config from '@/config.js';
import pagesJson from "@/pages.json";

export default {
    // 混入返回主页面工作台
    // #ifdef APP-NVUE
    mixins: [mixinCommon, mixinJump, mixinNvue, mixinVConsole, mixinRotateScreen],
    // #endif
    // #ifndef APP-NVUE
    // #ifdef MP-WEIXIN
    mixins: [mixinCommon, mixinJump, mixinRotateScreen],
    // #endif
    // #ifndef MP-WEIXIN
    mixins: [mixinCommon, mixinJump, mixinVConsole, mixinRotateScreen],
    // #endif
    // #endif
    
    data() {
        return {
            // 页面标题
            mv_navTitle: "",
            // 页面参数
            mv_options: null,
            // 是否开启骨架屏
            mv_skeleton: config.skeleton,
            // 窗口高度
            mv_windowHeight: 0,
            // 和原窗口高度相比是否变化
            mv_windowHeightChanged: false,
            // 窗口高度变化枚举：none/less/grow
            mv_windowHeightStatus: "none"
        }
    },
    
    /**
     * 计算型属性，不能在组件的data属性中直接使用，否则会抛出异常，可在onLoad及之后的周期中使用，如：initPageMore
     */
    computed: {
        /**
         * 兼容微信小程序
         */
        mc_options() {
            return this.mv_options;
        }
    },
    
    /**
     * 页面方法
     */
    methods: {
        /**
         * 设置页面标题
         * @param {Object} title
         */
        mixin_setPageTitle(title) {
            uni.setNavigationBarTitle({ title })
        },
        
        /**
         * 获取页面
         */
        mixin_getPage(upward = 1) {
            return this.$kit.getPage(upward)
        },
        
        /**
         * 窗口尺寸监听回调
         */
        _onWindowResize(res) {
            // 和原窗口高度相比是否变化
            if (this.vuex_systemInfo.windowHeight == res.size.windowHeight) {
                this.mv_windowHeightChanged = false;
            } else {
                this.mv_windowHeightChanged = true;
            }
            
            // 修改横竖屏切换标志
            if (res.size.windowHeight < res.size.windowWidth) {
                this.mixin_vertical = false;
            } else {
                this.mixin_vertical = true;
            }
            
            // 本次变化方向，变大还是变小
            if (res.size.windowHeight < this.mv_windowHeight) {
                this.mv_windowHeightStatus = "less";
            } else {
                this.mv_windowHeightStatus = "grow";
            }
            
            // 保存此次窗口高度
            this.mv_windowHeight = res.size.windowHeight;
        },
        
        /**
         * 初始化窗口高度
         */
        initWindowHeight() {
            // 初始化系统信息
            uni.getSystemInfo().then(res => {
                // 保存systemInfo
                // #ifdef VUE2
                let systemInfo = res[1];
                // #endif
                // #ifdef VUE3
                let systemInfo = res;
                // #endif
                
                this.$u.vuex('vuex_systemInfo', systemInfo);
                
                // 初始化页面高度
                this.mv_windowHeight = systemInfo.windowHeight;
                
                // 初始化横竖屏切换标志
                if (systemInfo.windowHeight < systemInfo.windowWidth) {
                    this.mixin_vertical = false;
                } else {
                    this.mixin_vertical = true;
                }
            })
            
            // 监听窗口尺寸变化事件
            uni.onWindowResize(this._onWindowResize)
        },
        
        /**
         * 初始化页面标题，为兼容h5集成设置
         * 注意：该方法不建议重写
         */
        mixin_initPage(options) {
            // 微信扫描跳转目录
            // #ifdef MP-WEIXIN
            if (options.q) {
                const url = decodeURIComponent(options.q);
                this.mv_options = this.$kit.getParams(url);
            } else {
                this.mv_options = options;
            }
            // #endif
            // #ifndef MP-WEIXIN
            this.mv_options = options;
            // #endif
            
            // 导航栏标题
            if (options.navTitle) {
                this.mv_navTitle = options.navTitle;
            }
            
            // #ifdef H5
            // 设置集成平台，主要为测试使用
            if (!this.$kit.isStrictEmpty(options.platform)) {
                this.$config.platform = options.platform;
                //welink平台集成导入华为js
                if (this.$config.platform == "welink") {
                    // 导入远程js
                    this.$kit.importJS("https://open-doc.welink.huaweicloud.com/docs/jsapi/2.0.6/hwh5-cloudonline.js");
                }
            }
            // #endif
        }
    },
    
    // 删除mixin_initMore方法，更多初始化可以放在页面onLoad方法
    // 注意：这是页面的混入，组件没有onLoad生命周期方法，因此这里的页面初始化不会在组件中被执行，同理下面的onShow和onUnload周期方法也不会在组件中有效
    onLoad(options) {
        // 全局设置当前页面，方便其他非页面js中直接调用页面绑定
        const app = getApp();
        if (app) {
            app.globalData.curPage = this;
        }
        
        // 初始化窗口高度
        this.initWindowHeight();
        // 初始化页面
        this.mixin_initPage(options);
    },
    
    /**
     * 原生tabbar导航使用
     * 
     * 因为uniapp的原生tabbar加载tab页面后会缓存当前页面，再次切换到该页面只会调用onShow
     * 如果不再onShow中设置标题，会导致标题不切换问题
     */
    onShow() {
        // 如果全局当前页面为null，将设置此页面为当前页面
        const app = getApp();
        if (app) {
            app.globalData.curPage = this;
        }
        
        // 解决pages.json中tabBar.list中text国际化设置（形如：%xxx%）无效问题
        const {
            tabBar: {
                list: pageList
            }
        } = pagesJson;
        const pagePaths = pageList.map(item => item.pagePath);
        const curPage = this.$kit.getPage();
        if (curPage) {
            const route = curPage.route;
            if (pagePaths.includes(route.getUrl())) {
                pageList.forEach((item, index) => {
                    if (item.text.startsWith("%") && item.text.endsWith("%") && item.text.length > 2) {
                        uni.setTabBarItem({
                            index,
                            text: this.$t(item.text.match(/%(\S*)%/)[1])
                        })
                    }
                })
            }
        }
        
        // 设置标题
        setTimeout(_ => {
            this.mixin_setPageTitle(this.mv_navTitle)
        }, 200)
    },
    
    // 卸载页面
    onUnload() {
        // 当页面销毁时，将当前页面对象值为null
        const app = getApp();
        if (app) {
            app.globalData.curPage = null;
        }
        
        // 取消监听窗口尺寸变化事件
        uni.offWindowResize(this._onWindowResize)
    }
}