/**
 * 网络请求，支持回调和Promise方式调用，并封装了添加、更新和删除操作的回退上级页面操作
 */
import mixinJump from "../common/jump.js";

export default {
    mixins: [mixinJump],
    /**
     * 混入方法
     */
    methods: {
        /**
         * 构建请求对象的数组，返回时已经开始执行，可通过then和catch处理
         */
        mixin_httpCall(request) {
            const _httpCall = (req) => {
                const {
                    method,
                    param
                } = req;
                if (param) {
                    if (this.$kit.isArray(param)) {
                        return method.apply(null, param);
                    }
                    return method.apply(null, [param]);
                }
                return method();
            }
            if (this.$kit.isArray(request)) {
                return Promise.all(request.map(req => _httpCall(req)));
            }
            return _httpCall(request)
        },

        /**
         * 请求结束
         */
        mixin_reqEnd(showLoading = true, exception = false) {
            if (showLoading) {
                uni.hideLoading();
                this.mv_loading = false;
            }
            // todo base/list/tab-list/tabbar-list中实现
        },

        /**
         * 用于简化http请求，如果通过callback方法回调，无需异常处理
         * 注意：该方法会显示加载模态框，如果只是单纯调用请使用mixin_httpCall方法
         */
        mixin_http(request, callback, showLoading = true) {
            // 显示加载模态
            if (showLoading) {
                this.$kit.showLoading(this.$t('common.waitMoment'));
                this.mv_loading = true;
            }
            
            // 如果存在回调方法，则不返回promise，类似uniapp的网络请求
            if (callback) {
                this.mixin_httpCall(request).then(res => {
                    if (this.$kit.checkStatus(res)) {
                        // 重置相关状态
                        this.mixin_reqEnd(showLoading);
                        
                        // 回调，这里已经检测网络状态，可直接处理业务数据
                        callback(res);
                    } else {
                        // 转入catch处理
                        return Promise.reject(res)
                    }
                }).catch(res => {
                    this.$kit.checkStatus(res, false);
                    this.mixin_reqEnd(true, true);
                })
            } else {
                // 不存在回调，则返回promise做后续处理
                return this.mixin_httpCall(request).then(res => {
                    // 重置相关状态
                    this.mixin_reqEnd(showLoading);
                    return res
                }).catch(res => {
                    // 重置相关状态
                    this.mixin_reqEnd(showLoading);
                    return res
                });
            }
        },
        
        /**
         * 请求成功，页面回调并返回上一页
         */
        mixin_http_back(request, method, ...params) {
            // mixin_back*的实现在page.js
            this.mixin_http(request, _ => this.mixin_back(method, ...params))
        },
        
        /**
         * 请求成功，已添加页面回调并返回上一页
         */
        mixin_http_backAdded(request, ...params) {
            this.mixin_http(request, _ => this.mixin_backAdded(...params))
        },
        
        /**
         * 请求成功，已更新页面回调并返回上一页
         */
        mixin_http_backUpdated(request, ...params) {
            this.mixin_http(request, _ => this.mixin_backUpdated(...params))
        },
        
        /**
         * 请求成功，已删除页面回调并返回上一页
         */
        mixin_http_backDeleted(request, ...params) {
            this.mixin_http(request, _ => this.mixin_backDeleted(...params))
        }
    }
}
