/**
 * 混入初始化网络请求数据，一般用于页面基础数据请求，不随页面刷新操作而重新请求（如需随页面刷新操作而重新请求数据，见 base 混入）
 */

import mixinHttp from './http.js';

export default {
    mixins: [mixinHttp],
    
    /**
     * 基础数据
     */
    data() {
        return {
            mv_initialized: false
        }
    },
    
    watch: {
        /**
         * 初始化后
         * @param {Object} newValue
         * @param {Object} oldValue
         */
        mv_initialized(newValue, oldValue) {
            // 前值false，后值true才会尝试调用其他业务数据，主要是列表数据，否则需要在初始化之后手动请求其他业务数据
            if (!oldValue && this.mv_initialized && this.mixin_loadData) {
                this.mixin_loadData();
            }
        }
    },

    methods: {
        /**
         * 初始化返回的列表业务数据
         * @param {Object} res
         */
        mixin_initData(res) {
            throw new Error('Component must implement mixin_initData method')
        },
        
        /**
         * 请求结束，重置加载状态
         * @param {Object} showLoading
         */
        mixin_initReqEnd(showLoading) {
            if (showLoading && !this.mixin_request) {
                uni.hideLoading();
            }
            
            // 初始化完成
            this.mv_initialized = true;
        },

        /**
         * 其他数据请求返回正常处理方法
         */
        mixin_initReqThen(res, showLoading = true) {
            // 重置状态
            this.mixin_initReqEnd(showLoading);
            
            if (this.$kit.checkStatus(res, false)) {
                this.mixin_initData(res)
            }
        },

        /**
         * 其他数据请求返回异常处理方法
         */
        mixin_initReqCatch(res, showLoading = true) {
            this.mixin_initReqEnd(showLoading);
            this.$kit.checkStatus(res, false);
        },

        /**
         * 返回是否需要网络请求初始化数据（也可在加载初始化数据前做准备工作）
         * true: 处理完成，不在执行后续初始化请求
         * false: 继续执行初始化请求
         */
        mixin_initLoadBofore() {
            return false
        },

        /**
         * 加载业务数据, 不建议重写（规则）
         */
        mixin_loadInitData(showLoading = true) {
            /**
             * 其他准备工作
             */
            if (this.mixin_initLoadBofore()) {
                // 判断不需要初始化，设置为已初始化
                this.mv_initialized = true
                return
            }

            if (!this.mixin_initRequest) {
                // 没有初始化请求的对象，也被认为不需要网络请求初始化数据，设置为已初始化
                this.mv_initialized = true
                return
            }
            
            if (showLoading) {
                uni.showLoading()
            }
            
            // 网络请求
            this.mixin_httpCall(this.mixin_initRequest).then(res => {
                this.mixin_initReqThen(res, showLoading)
            }).catch(res => {
                this.mixin_initReqCatch(res, showLoading)
            })
        }
    },

    /**
     * 加载页面钩子函数，可与页面内的onLoad一起执行，此处的钩子会先执行
     */
    onLoad(option) {
        // 加载初始化数据，如果未提供initRequests则直接返回
        // #ifdef MP-WEIXIN
        if (this.$kit.isAndroid()) {
            setTimeout(_ => this.mixin_loadInitData(), 100);
        } else {
            this.mixin_loadInitData()
        }
        // #endif
        // #ifndef MP-WEIXIN
        this.mixin_loadInitData()
        // #endif
    }
}
