/*!
 * @file httpClient.js
 * @brief 基于request封装客户端请求方法
 * Promise化小程序接口
 */

/*-------------------------url-----------------------*/

/**
 * 网络协议，正式版的小程序必须使用https协议
 */
const PROTOCOL = ["http", "https"][uni.$config.NPType];

/**
 * 服务端主机和端口
 */
const hostPort = `${uni.$config.host}:${uni.$config.ports[uni.$config.NPType]}`;

/**
 * 获取带协议的主机端口路径
 */
function hostUrl() {
    return `${PROTOCOL}://${hostPort}`;
}

/**
 * 获取带前缀的服务端基础路径
 */
function baseUrl(protocal = PROTOCOL) {
    return `${protocal}://${hostPort}/api`;
}

/**
 * 获取websocket连接的主机端口路径
 */
function baseSocketUrl() {
    return ["ws", "wss"][uni.$config.NPType] + `://${hostPort}`;
}

/**
 * 获取移动端带协议的主机端口路径
 */
function baseMobileUrl() {
    return `${PROTOCOL}://${uni.$config.host}:${uni.$config.mobilePorts[uni.$config.NPType]}`;
}

/**
 * 补全完整请求路径
 * @param {Object} path
 */
function buildUrl(path) {
    if (path.startsWith("http")) {
        return path;
    }
    return baseUrl() + path;
}

/*-------------------------headers-----------------------*/

/**
 * 表单参数模式
 */
const HEADER_X_WWW_FORM = {
    "content-type": 'application/x-www-form-urlencoded;charset=utf-8'
}

/**
 * 不带cookie，否则会出错的接口
 */
const HEADER_NO_COOKIE = {
    "noCookie": true
}

/**
 * 构建认证头部，用于未缓存认证信息时的连续网络请求。
 * 注：第一个请求获取token，后续请求可通过该方法构建认证头
 * @param {Object} token
 */
function buildAuthHeader(token) {
    /**
     * 支持全平台authorization认证
     */
    let authHeader = {}
    if (auth) {
        authHeader = {
            Authorization: auth
        }
    }

    /**
     * 支持非H5平台Cookie认证，H5平台不会缓存Cookie
     */
    // #ifndef H5
    if (res.header['Set-Cookie'] || res.header['Set-Cookie ']) {
        authHeader = {
            ...authHeader,
            Cookie: res.header['Set-Cookie'] || res.header['Set-Cookie '],
        }
    }
    // #endif

    return authHeader
}

/**
 * 根据自定义和缓存生成请求头
 */
function buildHeaders(custom, upload = false) {
    const headerCustom = {
        noCookie: false,
        ...custom
    }

    /**
     * 默认接收json数据格式
     */
    let headers = {
        "Accept": "application/json"
    }

    /**
     * 默认请求数据是json格式
     */
    // #ifdef H5
    if (!upload) { // h5文件上传不能设置content-type
        headers = {
            ...headers,
            "content-type": 'application/json'
        }
    }
    // #endif
    // #ifndef H5
    headers = {
        ...headers,
        "content-type": upload ? 'multipart/form-data' : 'application/json'
    }
    // #endif

    /**
     * 支持同源H5的Cookie认证和跨域非Cookie认证，不支持跨域Cookie认证
     */
    const headerCache = {
        Authorization: uni.getStorageSync('uni_id_token')
    }
    if (headerCache && headerCustom.noCookie) {
        // #ifndef H5
        headerCache.Cookie && delete headerCache.Cookie // 非H5不支持设置Cookie
        // #endif
        headerCache.Authorization && delete headerCache.Authorization
    }

    delete headerCustom.noCookie

    headers = {
        ...headers,
        ...headerCache,
        ...headerCustom
    };

    console.log("headers: ", headers);

    return headers;
}

/*-------------------------base api-----------------------*/

/**
 * UNI网络请求默认参数配置
 */
const DEFAULT_PARAMS = {
	// #ifdef H5
	"withCredentials": false // 2.6.15.20200421开始支持（H5）
	// #endif
}

/***
 * http GET请求
 * @param path
 * @param responseType : 'json'
 */
function GET(path, data, headers, conf) {
    const url = buildUrl(path)
    uni.$kit.log(`uni get: ${url}`);
    if (!uni.$u.test.empty(data)) {
        uni.$kit.log("payload", data);
    }

    // uni-app 对 API 进行了 Promise 封装，如无 success、fail、complete 参数，将以 Promise 返回数据
    return uni.request({
        ...DEFAULT_PARAMS,
        url,
        data,
        method: 'GET',
        header: buildHeaders(headers),
        ...conf
    }).then(res => {
        uni.$kit.log(`${path} response`, res[1]);
        return res[1];
    }).catch(err => {
        uni.$kit.error(`${path} catch`, err);
        return err
    })
}

/***
 * http GET请求
 * @param path
 * @param data
 * @param headers {'content-type': 'application/x-www-form-urlencoded;charset=utf-8'}
 * @param conf
 * @return responseType : 'json'
 */
function GET_FORM(path, data, headers = {}, conf = {}) {
    return GET(path, data, {
        ...headers,
        ...HEADER_X_WWW_FORM
    }, conf)
}

/***
 * http POST请求, 注意data不能是数组
 * @param path
 * @param responseType : 'json'
 */
function POST(path, data, headers, conf) {
    const url = buildUrl(path)
    uni.$kit.log(`uni post: ${url}`);
    if (!uni.$u.test.empty(data)) {
        uni.$kit.log("payload", data);
    }

    // uni-app 对 API 进行了 Promise 封装，如无 success、fail、complete 参数，将以 Promise 返回数据
    return uni.request({
        ...DEFAULT_PARAMS,
        url,
        data,
        method: 'POST',
        header: buildHeaders(headers),
        ...conf
    }).then(res => {
        uni.$kit.log(`${path} response`, res[1]);
        return res[1];
    }).catch(err => {
        uni.$kit.error(`${path} catch`, err);
        return err
    })
}

/**
 * PUT请求, 注意data不能是数组
 * @param {Object} path
 * @param {Object} data
 * @param {Object} headers
 */
function PUT(path, data, headers, conf) {
    const url = buildUrl(path)
    uni.$kit.log(`uni put: ${url}`);
    if (!uni.$u.test.empty(data)) {
        uni.$kit.log("payload", data);
    }

    // uni-app 对 API 进行了 Promise 封装，如无 success、fail、complete 参数，将以 Promise 返回数据
    return uni.request({
        ...DEFAULT_PARAMS,
        url,
        data,
        method: 'PUT',
        header: buildHeaders(headers),
        ...conf
    }).then(res => {
        uni.$kit.log(`${path} response`, res[1]);
        return res[1];
    }).catch(err => {
        uni.$kit.error(`${path} catch`, err);
        return err
    })
}

/**
 * http DELETE请求
 * @param {Object} path
 * @param {Object} data
 * @param {Object} headers
 */
function DELETE(path, data, headers, conf) {
    const url = buildUrl(path)
    uni.$kit.log(`uni delete: ${url}`);
    if (!uni.$u.test.empty(data)) {
        uni.$kit.log("payload", data);
    }

    // uni-app 对 API 进行了 Promise 封装，如无 success、fail、complete 参数，将以 Promise 返回数据
    return uni.request({
        ...DEFAULT_PARAMS,
        url,
        data,
        method: 'DELETE',
        header: buildHeaders(headers),
        ...conf
    }).then(res => {
        uni.$kit.log(`${path} response`, res[1]);
        return res[1];
    }).catch(err => {
        uni.$kit.error(`${path} catch`, err);
        return err
    })
}

/**
 * 单文件上传
 * @param {Object} path
 * @param {Object} formData
 * @param {Object} file
 * @param {Object} onProgress
 * @param {Object} headers
 * @param {Object} index
 */
function UPLOAD(path, formData, file, onProgress, headers, index = 0) {
    const url = buildUrl(path)
    const filePath = file.url
    uni.$kit.log(`uni upload: ${url}`);
    uni.$kit.log("payload filePath: ", filePath);
    if (!uni.$u.test.empty(formData)) {
        uni.$kit.log("payload formData: ", formData);
    }

    return new Promise((resolve, reject) => {
        const uploadTask = uni.uploadFile({
            ...DEFAULT_PARAMS,
            // #ifdef H5
            url: formData ? (url + "?fileName=" + formData.fileName) :
            url, // h5平台uniapp文件上传接口不识别表单文件名
            // #endif
            // #ifndef H5
            url,
            // #endif
            formData,
            filePath,
            name: 'file',
            header: buildHeaders(headers, true),
            success: (res) => {
                res.data = JSON.parse(res.data)
                uni.$kit.log("upload success res: ", res);
                resolve(res);
            },
            fail: (res) => {
                uni.$kit.log("upload fail res: ", res);
                reject(res);
            }
        })
        if (onProgress && uni.$kit.isFunction(onProgress)) {
            // onProgressUpdate 返回参数说明
            // 参数	类型	说明
            // progress	Number	上传进度百分比
            // totalBytesSent	Number	已经上传的数据长度，单位 Bytes
            // totalBytesExpectedToSend	Number	预期需要上传的数据总长度，单位 Bytes
            uploadTask.onProgressUpdate(res => {
                uni.$kit.log("upload onProgressUpdate res: ", res);
                onProgress(res, index);
            })
        }
    }).then(res => {
        uni.$kit.log("upload response: ", res);
        return res;
    }).catch(err => {
        uni.$kit.error("upload catch: ", err);
        return err
    })
}

/**
 * 多文件传输
 * @param {Object} path
 * @param {Object} formData
 * @param {Object} files
 * @param {Object} onProgress
 * @param {Object} headers
 */
function UPLOADS(path, formData, files, onProgress, headers) {
    uni.$kit.log(`uni uploads: ${path}`);
    uni.$kit.log("payload files: ", files);
    if (!uni.$u.test.empty(formData)) {
        uni.$kit.log("payload formData: ", formData);
    }
    
    return Promise.all(files.map((item, index) => {
        return UPLOAD(path, formData ? formData[index] : null, item, onProgress, headers, index)
    })).then(results => {
        let result = {
            statusCode: 200,
            data: {
                status: 200,
                message: "请求成功!",
                data: results.map(item => {
                    return item.data.data
                })
            }
        }
        uni.$kit.log("uploads response: ", result)
        return result
    }).catch(err => {
        uni.$kit.error(`${url} catch`, err)
        return err
    })
}

/**
 * 下载文件
 * @param {Object} path
 * @param {Object} filePath
 * @param {Object} onProgress
 * @param {Object} headers
 */
function DOWNLOAD(path, filePath, onProgress, headers) {
    const url = buildUrl(path)
    uni.$kit.log("uni download", url);
    if (filePath) {
        uni.$kit.log("save to filePath", filePath);
    }

    if (onProgress) {
        return new Promise((resolve, reject) => {
            const downloadTask = uni.downloadFile({
                ...DEFAULT_PARAMS,
                url,
                header: buildHeaders(headers),
                // #ifdef MP-WEIXIN
                filePath,
                // #endif
                success: (res) => {
                    uni.$kit.log("success res", res);
                    resolve(res);
                },
                fail: (res) => {
                    uni.$kit.log("fail res", res);
                    reject(res);
                }
            });
            downloadTask.onProgressUpdate(res => {
                onProgress(res, downloadTask);
            });
        });
    } else {
        return uni.downloadFile({
            ...DEFAULT_PARAMS,
            url,
            // #ifdef MP-WEIXIN
            filePath,
            // #endif
            header: buildHeaders(headers)
        }).then(res => {
            uni.$kit.log("download response", res[1]);
            return res[1];
        }).catch(err => {
            uni.$kit.error("download catch", err);
            return err
        })
    }
}

export default {
    // base api
    GET,
    GET_FORM,
    POST,
    PUT,
    DELETE,
    UPLOAD,
    UPLOADS,
    DOWNLOAD,

    // url
    url: {
        baseUrl,
        baseSocketUrl,
        baseMobileUrl,
        buildUrl
    },

    // header
    header: {
        noCookie: HEADER_NO_COOKIE,
        x3wForm: HEADER_X_WWW_FORM,
        buildAuthHeader,
        buildHeaders
    }
}
