/**
 * Vuex 是一个专为 Vue.js 应用程序开发的状态管理模式。它采用集中式存储管理应用的所有组件的状态，并以相应的规则保证状态以一种可预测的方式发生变化。
 * 注意：Vuex 并不限制你的代码结构。但是，它规定了一些需要遵守的规则
 * （1）应用层级的状态应该集中到单个 store 对象中
 * （2）提交 mutation 是更改状态的唯一方法，并且这个过程是同步的
 * （3）异步逻辑都应该封装到 action 里面
 * （4）严模式下，对表单v-model属性使用双向绑定的计算属性。参考：https://vuex.vuejs.org/zh/guide/forms.html
 * 参考链接：https://vuex.vuejs.org/zh/
 */

/**
 * 如果所有状态会集中后得到一个比较大的对象，可考虑模块化store
 * 参考链接：https://vuex.vuejs.org/zh/guide/modules.html
 */
import modules from '@/store/modules'
import plugins from '@/store/plugins'

// 全局对象
import getters from '@/store/getters.js'
import mutations from '@/store/mutations.js'
import actions from '@/store/actions.js'

/**
 * 本地storage缓存的数据
 */
let lifeData = {};

try {
    // 尝试获取本地是否存在lifeData变量，第一次启动APP时是不存在的
    lifeData = uni.getStorageSync('lifeData');
} catch (e) {

}

/**
  * 原理同Vue的data属性，防止被共享互相污染
  * 如果我们使用一个纯对象来声明模块的状态，那么这个状态对象会通过引用被共享，导致状态对象被修改时store或模块间数据互相污染的问题
  */
function state() {
    return {
        /**
         * 如果上面从本地获取的lifeData对象下有对应的属性，就赋值给state中对应的变量
         * 加上vuex_前缀，是防止变量名冲突，也让人一目了然
         */
        // 第一次启动标志
        vuex_launchFlag: !!lifeData.vuex_launchFlag,
        
        // 设备系统数据
        vuex_systemInfo: null,
        
        // 主题
        vuex_theme: lifeData.vuex_theme || 'primary',
        
        /**
         * tabBar，请保持pagePath页面路径和pages.json的tabBar字段中的一致
         * 注意：采用cu-tabbar和原生tabbar系统结合可是页面复杂度降低，但是由于每个页面
         *      都有cu-tabbar控件，导致第一遍切换页面时cu-tabbar出现闪烁现象，因此如果不能接受闪烁
         *      请使用uniapp原生tabbar系统，但是原生tabbar系统不支持字体图标库
         */
        vuex_tabBar: lifeData.vuex_tabBar || []
    }
}

// #ifndef VUE3
import Vue from 'vue'
import Vuex from 'vuex'
Vue.use(Vuex)
const store = new Vuex.Store({
    /**
     * 说明：在严格模式下，无论何时发生了状态变更且不是由 mutation 函数引起的，将会抛出错误。这能保证所有的状态变更都能被调试工具跟踪到。
     * 注意：不要在发布环境下启用严格模式！严格模式会深度监测状态树来检测不合规的状态变更——请确保在发布环境下关闭严格模式，以避免性能损失。
     */
    strict: process.env.NODE_ENV !== 'production',
    state,
    getters,
    mutations,
    actions,
	modules, // 合入vuex_user、vuex_settings空间
    plugins
})
// #endif

// #ifdef VUE3
import { createStore } from 'vuex'
const store = createStore({
    state,
    getters,
    mutations,
    actions,
	modules,
    plugins
})
// #endif

export default store