/**
 * 说明：更改 Vuex 的 store 中的状态的唯一方法是提交 mutation，Mutation 必须是同步函数。
 * Vuex 中的 mutation 非常类似于事件：每个 mutation 都有一个字符串的 事件类型 (type) 和 一个 回调函数 (handler)
 * 注意：Mutation 需遵守 Vue 的响应规则
 * （1）最好提前在你的 store 中初始化好所有所需属性
 * （2）当需要在对象上添加新属性时，你应该使用 Vue.set(obj, 'newProp', 123), 或者以新对象替换老对象
 * 使用方式：this.$store.commit('xxx') or mapMutations辅助函数
 */

// 需要永久存储，且下次APP启动需要取出的，在state中的变量名
let saveStateKeys = [
    'vuex_launchFlag',
    'vuex_theme',
    'vuex_settings',
    'vuex_tabBar',
    'vuex_user'
];

/**
 * 保存变量到本地存储中
 * @param {Object} key
 * @param {Object} value
 */
const saveLifeData = function(key, value) {
    // 判断变量名是否在需要存储的数组中
    if (saveStateKeys.indexOf(key) != -1) {
        // 获取本地存储的lifeData对象，将变量添加到对象中
        let tmp = uni.getStorageSync('lifeData');
        // 第一次打开APP，不存在lifeData变量，故放一个{}空对象
        tmp = tmp || {};
        tmp[key] = value;
        // 执行这一步后，所有需要存储的变量，都挂载在本地的lifeData对象中
        uni.setStorageSync('lifeData', tmp);
    }
}

/**
 * 清除本地存储
 * @param {Object} state 
 */
const clearLifeData = function(state) {
    // 重置数据
    const lifeData = {
        // 第一次启动标志
        vuex_launchFlag: state.vuex_launchFlag,
        
        // 清除配置信息
        vuex_theme: state.vuex_theme,
        
        // 系统配置
        vuex_settings: state.vuex_settings,
        
        // 底部导航栏
        vuex_tabBar: [],
        
        // 清除用户信息
        vuex_user: {}
    };

    // 执行这一步后，所有需要存储的变量，都挂载在本地的lifeData对象中
    uni.setStorageSync('lifeData', lifeData);
}

export default {
    /**
     * 所有的vuex变量通过$uStore保存，将payload.value保存payload.name中，注意name可能是多层级的属性
     * 被调用方式：this.$store.commit('$uStore', payload)
     * @param {Object} state
     * @param {Object} payload
     */
    $vuexMutation(state, payload) {
        // 判断是否多层级调用，state中为对象存在的情况，诸如user.info.score = 1
        let nameArr = payload.name.split('.');
        let saveKey = '';
        let len = nameArr.length;
        if (len >= 2) {
            let obj = state[nameArr[0]];
            for (let i = 1; i < len - 1; i++) {
                obj = obj[nameArr[i]];
            }
            obj[nameArr[len - 1]] = payload.value;
            saveKey = nameArr[0];
        } else {
            // 单层级变量，在state就是一个普通变量的情况
            state[payload.name] = payload.value;
            saveKey = payload.name;
        }

        // 保存变量到本地，见顶部函数定义
        saveLifeData(saveKey, state[saveKey])
    },

    /**
     * 重置vuex变量
     * @param {Object} state
     */
    $uClearVuex(state) {
        // 清除state属性变量
        // todo
        
        // 清除本地缓存
        clearLifeData(state);
    }
}
